#!/usr/bin/env perl

# cycles and starts jobs

use strict;
use warnings;

use Data::Dumper;

use YAML::Syck;
use FindBin;
use lib "$FindBin::Bin/lib";
use App::Locker;
use App::Logger;
use App;
use Time::HiRes qw/sleep/;

# config
my $daemon = YAML::Syck::LoadFile("$FindBin::Bin/etc/daemon/config.yml");

# settings
$Data::Dumper::Indent = 0;

# logger
my $logger = App::Logger->new(
  file => $FindBin::Bin.'/'.$daemon->{ctrl}->{log}->{path}.'/'.$daemon->{ctrl}->{log}->{filename},
  debug => defined $daemon->{ctrl}->{debug} ? $daemon->{ctrl}->{debug} : 1,
  verbosity => defined $daemon->{ctrl}->{verbosity} ? $daemon->{ctrl}->{verbosity} : 2,
);

$logger->info('start');

# control
$main::term = 0;
my $sig_term = sub { $main::term = 1; };
local $SIG{INT}  = $sig_term;
local $SIG{TERM} = $sig_term;

eval {

  # settings
  my $sleep = $daemon->{ctrl}->{sleep} || 10;
  my $dryrun = defined $daemon->{ctrl}->{dryrun} ? $daemon->{ctrl}->{dryrun} : 0;

  $logger->debug("daemon ctrl sleep |$sleep|");
  $logger->debug("daemon ctrl dryrun |$dryrun|");

  my $job_cnt;

  LOOP:
  while ( !$main::term ) {

    $logger->info('alive and looping', 2);

    # config
    my $payload = YAML::Syck::LoadFile("$FindBin::Bin/etc/jobs/payload.yml");

    my @jobs = sort keys %{$payload->{jobs}};
    $logger->debug('jobs ' . Dumper( \@jobs ) );

    # maybe init /job_cnt/
    unless ( defined $job_cnt ) {
      $logger->info('init job_cnt');
      $job_cnt = int(rand( scalar @jobs ));
      $logger->debug("job_cnt |$job_cnt|");
    }

    # set /job_cnt/
    $job_cnt++;
    $job_cnt = 0 if $job_cnt > $#jobs;
    $logger->debug("job_cnt |$job_cnt|");

    # get /job_id/
    my $job_id = $jobs[ $job_cnt ];
    $logger->info("job_id |$job_id|", 1);

    # workout job
    my $nolockmsg = "nolock\n";
    eval {

      $logger->info("attempt to get lock on job_id |$job_id|", 1);
      my $fh_lock = get_lock(
        $FindBin::Bin.'/'.$daemon->{jobs}->{lock}->{path}.'/'.$job_id.$daemon->{jobs}->{lock}->{ext},
        $nolockmsg,
      );
      $logger->info('lock obtained', 1);

      $logger->info("got lock on job_id |$job_id|");
      $logger->info('start processing', 1);

      # job config
      my $job = $payload->{jobs}->{ $job_id };
      $logger->debug('job ' . Dumper( $job ) );

      {
        # settings
        local $Data::Dumper::Indent = 2;

        # new App for job
        $logger->info('app App->new', 2);
        my $app = App->new(
          job => $job,
        );

        # run job
        unless ( $dryrun ) {
          $logger->info('app start', 1);
          my $ret = $app->run();
          if ( $ret ) {
            $logger->info('app error', 1);
          }
          else {
            $logger->info('app done', 1);
          }
        }
      }

      $logger->info('done processing', 1);
      $logger->info("attempt to release lock from job_id |$job_id|", 1);
      release_lock( $fh_lock );
      $logger->info("lock released from job_id |$job_id|", 1);

    };
    # no lock
    if ( $@ eq $nolockmsg ) {
      $logger->info("no lock on job_id |$job_id|", 1);
    }
    # error
    elsif ( $@ ) {
      my $syntax_error_message = $@;
      chomp $syntax_error_message;
      $logger->fatal("workout on job_id |$job_id| died |$syntax_error_message|");
      die $syntax_error_message;
    }

    # no SIG via main::term received
    if ( !$main::term ) {
      $logger->info("sleep |$sleep|", 1);
      sleep $sleep;
      $logger->info('awake', 2);
      $logger->info('about to loop', 2);
    }
    # SIG via main::term received
    else {
      $logger->warning('SIG via main::term received', 2);
    }

  } # while ( !$main::term )

}; # eval
if ( $@ ) {
  my $syntax_error_message = $@;
  chomp $syntax_error_message;
  $logger->fatal("daemon died |$syntax_error_message|");
  die $syntax_error_message;
}
else {
  $logger->info('daemon gracefully stopped');
}

$logger->info('stop');

exit(0);

