package App;

# workout job

use strict;
use warnings;

use Carp;
use Data::Dumper;

use YAML::Syck;
use FindBin;
use lib "$FindBin::Bin";
use App::Logger;
use App::Type::File_JSON;
use App::Type::HTTP_GET_JSON;
use App::Type::OAIHarvest_DC;
use App::Type::OAIHarvest_LOM;
use App::Type::dummy;
use List::MoreUtils qw/none/;
use Time::HiRes qw/sleep/;

sub new {
  my ($class, %args) = @_;

  my $self = {};
  # mandatory
  $self->{_job} = $args{job} or croak "no job given";

  bless($self, $class);

  $self->_init();
  $self->_sanity();

  return $self;
}

sub _init {
  my ($self) = @_;
  croak "_init is private" if caller() ne __PACKAGE__;

  # new logger
  $self->{_logger} = App::Logger->new(
    file => $FindBin::Bin.'/'.$self->{_job}->{cfg}->{log}->{path}.'/'.$self->{_job}->{cfg}->{log}->{filename},
    debug => defined $self->{_job}->{cfg}->{debug} ? $self->{_job}->{cfg}->{debug} : 1,
    verbosity => defined $self->{_job}->{cfg}->{verbosity} ? $self->{_job}->{cfg}->{verbosity} : 2,
  );

  return;
}

sub _sanity {
  my ($self) = @_;
  croak "_sanity is private" if caller() ne __PACKAGE__;

  ## settings
  # shortcut
  my $logger = $self->{_logger};

  # config
  my $sanity = YAML::Syck::LoadFile("$FindBin::Bin/etc/jobs/sanity.yml");

  $logger->info('sanity check start', 2);
  $logger->debug('_job ' . Dumper( $self->{_job} ) );

  croak "sanity check died, unknown job type |$self->{_job}->{type}|"
    if none { $_ eq $self->{_job}->{type} } ( keys %{$sanity->{cfgs}} );

  # sanity check job cfg
  _sanity_check( $sanity->{cfgs}->{ $self->{_job}->{type} }, $self->{_job}->{cfg} );

  $logger->info('sanity check done', 2);

  return;
}

# return: SCALAR; 0:no error, 1:error
sub run {
  my ($self) = @_;

  ## settings
  # shortcut
  my $logger = $self->{_logger};

  $logger->info('App start');

  # to monitor failure of eval block
  # this method shall not die/croak
  my $error;

  eval {

    $logger->info("sleep pre |$self->{_job}->{cfg}->{sleep}->{pre}|", 1);
    sleep $self->{_job}->{cfg}->{sleep}->{pre};

    my $type = $self->{_job}->{type};
    my $class = 'App::Type::'.$type;

    # new App::Type::/type/
    $logger->info('work App::Type::'.$type.'->new', 2);
    my $work = $class->new(
      cfg => $self->{_job}->{cfg},
    );

    $logger->info('work start', 1);
    $work->run();
    $logger->info('work done', 1);

    # no SIG via main::term received
    if ( !$main::term ) {
      $logger->info("sleep post |$self->{_job}->{cfg}->{sleep}->{post}|", 1);
      sleep $self->{_job}->{cfg}->{sleep}->{post};
    }

  }; # eval
  if ( $@ ) {
    my $syntax_error_message = $@;
    chomp $syntax_error_message;
    $error++;
    $logger->error("App died |$syntax_error_message|");
  }

  $logger->info('App done');

  return $error ? 1 : 0;
}

sub _sanity_check {
  my ($match, $value, $store) = @_;

  # traversal, recursion
  if ( ref( $match ) eq 'HASH' ) {

    # traverse level
    for my $key ( keys %{$match} ) {
      # remember level
      push @$store, $key;
      # recurse deeper
      _sanity_check( $match->{ $key }, $value->{ $key }, $store );
      # forget level
      pop @$store;
    }

  }
  # check
  else {
    my $id = join('/', @$store);
    my $re = qr{$match};
    croak "sanity check died, missing job cfg |$id|"
      unless defined $value;
    croak "sanity check died, invalid job cfg |$id|, value |$value| should match |$match|"
      unless $value =~ m{$re};
  }
}

1;

__END__

