package OAI_DC_xLang;
 
use strict;
use warnings;
use base qw( XML::SAX::Base );
use Carp qw( carp );
use Data::Dumper;
our $VERSION = "1.0";
 
use constant {
  XMLNS_DC => 'http://purl.org/dc/elements/1.1/',
  XMLNS_OAIDC => 'http://www.openarchives.org/OAI/2.0/oai_dc/',
};
 
our @OAI_DC_ELEMENTS = qw(
    title 
    creator 
    subject 
    description 
    publisher 
    contributor 
    date
    type
    format
    identifier
    source
    language
    relation
    coverage
    rights
);

 
our $AUTOLOAD;

 
sub new {
    my ( $class, %opts ) = @_;
    my $self = bless \%opts, ref( $class ) || $class;
    foreach ( @OAI_DC_ELEMENTS ) { 
        $self->{ $_ } = []; 
    }
    return( $self );
}

 
## rather than right all the accessors we use AUTOLOAD to catch calls
## valid element names as methods, and return appropriately as a list
 
sub AUTOLOAD {
    my $self = shift;
    my $sub = lc( $AUTOLOAD );
    $sub =~ s/.*:://;
    if ( grep /$sub/, @OAI_DC_ELEMENTS ) {
        if ( wantarray() ) { 
            return( @{ $self->{ $sub } } );
        } else { 
            return( $self->{ $sub }[0] );
        }
    }
}
 
## SAX handlers
 
sub start_element {
    my ( $self, $element ) = @_;
    my $elname = $element->{ LocalName };
    my $ns = $element->{ NamespaceURI };
    my $xml_lang = '{http://www.w3.org/XML/1998/namespace}lang';

    if ( ($element->{ NamespaceURI } eq XMLNS_OAIDC) and ($elname eq "dc") ) {
        $self->{ _insideRecord } = 1}
    elsif ( $element->{ NamespaceURI } ne XMLNS_DC ) {
        carp "what is ".$element->{ Name }."?";
        return undef;
      }
     elsif ( $element->{ Attributes }{ $xml_lang }{ Value } ) {
        $self->{ _lang } = $element->{ Attributes }{ $xml_lang }{ Value };
        $self->{ chars } = "";
      }
    elsif ( grep /$elname/, @OAI_DC_ELEMENTS ) {
        $self->{ chars } = "";
        $self->{ _lang } = undef }
    else {
        carp "what is $elname?"}
}
 
sub end_element {
    my ( $self, $element ) = @_;
    my $elname = $element->{ LocalName };
 
    if ( ($element->{ NamespaceURI } eq XMLNS_OAIDC) and ($elname eq "dc") ) {
        $self->{ _insideRecord } = 0}
    elsif ( $element->{ NamespaceURI } ne XMLNS_DC ) {
        return undef}
    elsif ( grep /$elname/, @OAI_DC_ELEMENTS ) {   # o.k.
        if ( $self->{ _lang } ) { # o.k. with xml:lang attribute
            my $ob = {
                $self->{ _lang } => $self->{ chars }
            };
            push( @{ $self->{ $elname } }, $ob ); 
        } else {
            push( @{ $self->{ $elname } }, $self->{ chars } );
        }
        $self->{ chars } = undef;
      }
    elsif ( $self->{ chars } =~ /\S/ ) {
        carp "unassigned content: ".$self->{ chars };
      }
}
 
sub characters {
    my ( $self, $characters ) = @_;
    $self->{ chars } .= $characters->{ Data } if $self->{ _insideRecord };
}
 
1;
