package OAI_LOM_invenio;
 
use strict;
use warnings;
use base qw( XML::SAX::Base );
use Carp qw( carp );
use Data::Dumper;
our $VERSION = "0.1";
 
use constant {
  XMLNS_DC => 'http://purl.org/dc/elements/1.1/',
  XMLNS_OAIDC => 'http://www.openarchives.org/OAI/2.0/oai_dc/',
  
  XMLNS_LOM_de => 'http://purl.org/dc/elements/1.1/',
  XMLNS_OAI_LOM_de => 'https://www.oerbw.de/hsoerlom',
  DEFAULT_LANGUAGE => ['de'],
};
 
# general lifecycle metametadata technical educational rights classification relation annotation catalog
our @OAI_LOM_ELEMENTS_FIRST_ORDER = qw(
    technical
 );

our @OAI_LOM_ELEMENTS_TECHNICAL_LEAF = qw(
    format
    size
    location
 );
#########
## taxonpath aggregationlevel
##
## origin at zoer: taxonPath aggregationLevel
#########
 
# our $AUTOLOAD;

sub new {
    my ( $class, %opts ) = @_;
    my $self = bless \%opts, ref( $class ) || $class;
    $self->{ format } = $self->{ size } = $self->{ location } = '';
    $self->{ id } = $self->{ entry } = $self->{ vcard } = '';
    $self->{ title } = $self->{ description } = $self->{ catalog } = $self->{ entry } = $self->{ language } = '';
    $self->{ identifier } = [];
    $self->{ keyword } = [];
    $self->{ _tagStack } = [];
    $self->{ langstring } = undef;
    $self->{ source } = $self->{ value } = '';
    $self->{ _insideMetadata } = 0;
    $self->{ _insideLom } = $self->{ _insideGeneral } = $self->{ _insideLifecycle } = 
        $self->{ _insideMetaMetadata } = $self->{ _insideTechnical } = 
        $self->{ _insideEducational } = $self->{ _insideRights } = 
        $self->{ _insideClassification } = $self->{ _insideClassificationPurpose } = $self->{ _insideStatus } = 0;
    $self->{ _insideDescription } = $self->{ _insideTitle } = $self->{ _insideAggregationLevel } = $self->{  _insideRole } = $self->{   _insideMetametadata } = 0;
    $self->{ _insideLearningResourceType } = $self->{  _insideCopyrightandotherrestrictions } = $self->{  _insideTaxon } = 0;
    return( $self );
}

# sub new {
#    my ( $class, %opts ) = @_;
#    my $self = bless \%opts, ref( $class ) || $class;
#    # foreach ( @OAI_LOM_ELEMENTS_FIRST_ORDER ) { 
#    foreach ( @OAI_LOM_ELEMENTS_LEAF ) { 
#       $self->{ $_ } = []; 
#    }
#    return( $self );
#}

 
## rather than right all the accessors we use AUTOLOAD to catch calls
## valid element names as methods, and return appropriately as a list
 
#sub AUTOLOAD {
#    my $self = shift;
#    my $sub = lc( $AUTOLOAD );
#    $sub =~ s/.*:://;
#    # if ( grep /$sub/, @OAI_LOM_ELEMENTS_FIRST_ORDER ) {
#    if ( grep /$sub/, @OAI_LOM_ELEMENTS_LEAF ) {
#        if ( wantarray() ) { 
#            return( @{ $self->{ $sub } } );
#        } else { 
#            return( $self->{ $sub }[0] );
#        }
#    }
#}
sub format {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ format } = $element; }
    return( $self->{ format } );
}

sub size {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ size } = $element; }
    return( $self->{ size } );
}

sub location {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ location } = $element; }
    return( $self->{ location } );
}

sub title {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ title } = $element; }
    return( $self->{ title } );
}

sub description {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ description } = $element; }
    return( $self->{ description } );
}

sub version {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ version } = $element; }
    return( $self->{ version } );
}

sub vcard {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ vcard } = $element; }
    return( $self->{ vcard } );
}

sub datetime {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ datetime } = $element; }
    return( $self->{ datetime } );
}

#sub identifier {
#    my ( $self, $element ) = @_;
#    if ( $element ) { $self->{ identifier } = $element; }
#    return( $self->{ identifier } );
#}

#sub catalog {
#    my ( $self, $element ) = @_;
#    if ( $element ) { $self->{ catalog } = $element; }
#    return( $self->{ catalog } );
#}

# sub entry {
#     my ( $self, $element ) = @_;
#     if ( $element ) { $self->{ entry } = $element; }
#     return( $self->{ entry } );
#}

# sub keyword {
#    my ( $self, $element ) = @_;
#     if ( $element ) { $self->{ keyword } = $element; }
#     return( $self->{ keyword } );
#}

#sub langstring {
#    my ( $self, $element ) = @_;
#    if ( $element ) { $self->{ langstring } = $element; }
#    return( $self->{ langstring } );
#    # return( @{ $self->{ langstring } } );
#}

sub lomGeneral {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomGeneral } = $element; }
    return( $self->{ lomGeneral } );
}
sub lomLifecycle {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomLifecycle } = $element; }
    return( $self->{ lomLifecycle } );
}
sub lomMetametadata {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomMetametadata } = $element; }
    return( $self->{ lomMetametadata } );
}
sub lomTechnical {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomTechnical } = $element; }
    return( $self->{ lomTechnical } );
}
sub lomEducational {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomEducational } = $element; }
    return( $self->{ lomEducational } );
}
sub lomRights {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomRights } = $element; }
    return( $self->{ lomRights } );
}

sub lomClassification {
    my ( $self, $element ) = @_;
    if ( $element ) { $self->{ lomClassification } = $element; }
    return( $self->{ lomClassification } );
}

sub lom {
    my ( $self ) = @_;
    my $lom = {
        general => $self->lomGeneral(),
        lifecycle => $self->lomLifecycle(),
        metametadata => $self->lomMetametadata(),
        technical => $self->lomTechnical(),
        educational => $self->lomEducational(),
        rights => $self->lomRights(),
        classification => $self->lomClassification(),
    };
    return( $lom );
}

sub trim {
    my ( $txt ) = @_;
    $txt =~ s/^\s+|\s+$|\R//g;
    return( $txt );
}

sub today {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    my $nice_timestamp = sprintf ( "%04d%-02d-%02d", $year+1900,$mon+1,$mday);
    return $nice_timestamp;
}

## SAX handlers
sub start_element {
    my ( $self, $element ) = @_;
    my $xml_lang = '{http://www.w3.org/XML/1998/namespace}lang';
    #unless ( $element->{ NamespaceURI } eq XMLNS_OAI_LOM_de ) {
    #    $self->SUPER::start_element($element) if $self->{ fwdAll } or $self->{ _insideMetadata };
    #    return;
    #}

    my $tagName = $element->{ LocalName };
    push( @{$self->{ _tagStack }}, $tagName );
    if ( $tagName eq 'metadata' ) {
	    $self->{ _insideMetadata } = 1;
    }
    elsif ( $tagName eq 'lom' ) {
	    $self->{ _insideLom } = 1;
    }
    elsif ( $tagName eq 'about' ) {
	    $self->{ _insideAbout } = 0;
    } 
    ##### General #####
    elsif ( $tagName eq 'general') {
        delete $self->{ langstringArray };
        delete $self->{ languages };
        $self->{ _insideGeneral } = 1;
    }
    elsif ( $tagName eq 'title') {
        delete $self->{ langstringArray };
        $self->{ _insideTitle } = 1;
    }
    elsif ( $tagName eq 'language') {
        $self->{ _insideLanguage } = 1;
    }
    elsif ( $tagName eq 'identifier') {
        $self->{ _insideIdentifier } = 1;
    }
    elsif ( $tagName eq 'catalog') {
        $self->{ _insideCatalog } = 1;
    }

    elsif ( $tagName eq 'keyword') {
        $self->{ _insideEntry } = 1;
    }
    elsif ( $tagName eq 'aggregationlevel') {
        $self->{ _insideAggregationLevel } = 1;
    }

    ##### Lifecycle #####
    elsif ( $tagName eq 'lifecycle') {
        delete $self->{ langstringArray }; 
        $self->{ _insideLifecycle } = 1;
    }
    elsif ( $tagName eq 'version') {
        $self->{ _insideVersion } = 1;
    }
    elsif ( $tagName eq 'status') {
        $self->{ _insideStatus } = 1;
    }

    ##### Metametadata #####
    elsif ( $tagName eq 'metametadata') {
        delete $self->{ langstringArray };
        $self->{ _insideMetametadata } = 1;
    }
    elsif ( $tagName eq 'contribute') {
        delete $self->{ lomMetametadataContributeEntity };
        delete $self->{ lomLifecycleContributeEntity };
        delete $self->{ lomMetametadataContributeCentity };
        delete $self->{ lomLifecycleContributeCentity };        
        $self->{ _insideContribute } = 1;
    }

    ##### Technical #####
    elsif ( $tagName eq 'technical') {
        delete $self->{ langstringArray };
        $self->{ _insideTechnical } = 1;
    }
    elsif ( $tagName eq 'format') {
        $self->{ _insideFormat } = 1;
    } 
    elsif ( $tagName eq 'size') {
        $self->{ _insideSize } = 1;
    } 
    elsif ( $tagName eq 'location') {
        $self->{ _insideLocation } = 1;
    }
    elsif ( $tagName eq 'duration' ) {
        $self->{ _insideDuration } = 1;
    }
    # non standard element 
    elsif ( $tagName eq 'thumbnail' ) {
        $self->{ _insideThumbnail } = 1;
    }
    # non standard element: description of the thumbnail
    elsif ( $tagName eq 'licenses' ) {
        $self->{ _insideLicenses } = 1;
    }

    ##### Educational #####
    elsif ( $tagName eq 'educational') {
        delete $self->{ langstringArray };
        delete $self->{ lomEducationalLearningResourceType }; # since change to hash a delete it here
        $self->{ _insideEducational } = 1;
    }
    elsif ( $tagName eq 'learningresourcetype') {
        $self->{ _insideLearningResourceType } = 1;
        delete $self->{ langstringArray } ; # here delete is a must
        $self->{ langstring } = ''; 
    }

    ##### Rights #####
    elsif ( $tagName eq 'rights') {
        delete $self->{ langstringArray };
        $self->{ _insideRights } = 1;
    }
    elsif ( $tagName eq 'copyrightandotherrestrictions') {
        $self->{ _insideCopyrightandotherrestrictions } = 1;
    }

    ##### Classification #####
    # the IEEE allows multiple classification and UIBK will use two #
    elsif ( $tagName eq 'classification') {
        delete $self->{ langstringArray };
        $self->{ _insideClassification } = 1;
    }
    elsif ( $tagName eq 'purpose') {
        $self->{ _insidePurpose } = 1;
    }
    elsif ( $tagName eq 'taxonpath') {
        delete $self->{ lomClassificationTaxonpath };
        delete $self->{ lomClassificationTaxonpathTaxon };
        $self->{ _insideTaxonpath } = 1;
    }
    elsif ( $tagName eq 'taxon') {
        delete $self->{ langstringArray };
        $self->{ _insideTaxon } = 1;
    }

    ##### used in several Elements #####
    elsif ( $tagName eq 'langstring' ) {
        if ( $element->{ Attributes }{ $xml_lang }{ Value } ) {
            $self->{ _lang } = $element->{ Attributes }{ $xml_lang }{ Value };
            $self->{ chars } = "";
        }
        $self->{ _insideLangstring } = 1;
    }
    elsif ( $tagName eq 'source') {
        $self->{ _insideSource } = 1;
    }
    elsif ( $tagName eq 'value') {
        $self->{ _insideValue } = 1;
    }
    elsif ( $tagName eq 'entry') {
        delete $self->{ langstringArray };
        $self->{ _insideEntry } = 1;
    }
    elsif ( $tagName eq 'description') {
        delete $self->{ langstringArray };
        $self->{ _insideDescription } = 1;
    }
    elsif ( $tagName eq 'role') {
        $self->{ _insideRole } = 1;
    }
    # old without vcard
    elsif ( $tagName eq 'entity') {
        $self->{ _insideEntity } = 1;
    }
    elsif ( $tagName eq 'centity') {
        $self->{ _insideCentity } = 1;
    }    
    elsif ( $tagName eq 'vcard') {
        $self->{ _insideVcard } = 1;
    }    
    elsif ( $tagName eq 'date') {
        $self->{ _insideDate } = 1;
    }
    elsif ( $tagName eq 'datetime') {
        $self->{ _insideDatetime } = 1;
    }
    elsif ( $tagName eq 'id') {
        $self->{ id } = '';
        $self->{ _insideId } = 1;
    }
    # non standard element: location of the thumbnail, licenses for thumbnail
    elsif ( $tagName eq 'url' ) {
        $self->{ url } = '';
        $self->{ _insideUrl } = 1;
    }
    else {
        carp "who am I SE? ($tagName)";
        #return $self->SUPER::start_element($element);
    };
    #return $self->SUPER::start_element($element) if $self->{ fwdAll };
}

sub end_element {
    my ( $self, $element ) = @_;
    # unless ( $element->{ NamespaceURI } eq XMLNS_OAI_LOM_de ) {
    #     $self->SUPER::end_element($element) if $self->{ fwdAll } or $self->{ _insideMetadata };
    #   return;
    #}
    
    pop( @{$self->{ _tagStack }} );
    my $tagName = $element->{ LocalName };
    if ( $tagName eq 'metadata' ) {
	    $self->{ _insideMetadata } = 0;
        delete $self->{ _insideLom };
    }
    elsif ( $tagName eq 'lom' ) {
	    $self->{ _insideLom } = 0;
        delete $self->{ _insideGeneral };
        delete $self->{ _insideLifecycle };
        delete $self->{ _insideMetametadata };
        delete $self->{ _insideTechnical };
        delete $self->{ _insideEducational };
        delete $self->{ _insideRights };
        delete $self->{ _insideClassification };
        # General
        delete $self->{ _insideTitle };
        delete $self->{ _insideDescription };
        # Metametadata
        delete $self->{ _insideMetaMetadata };
        # Technical
        delete $self->{ _insideFormat };
        delete $self->{ _insideSize };
        delete $self->{ _insideLocation };

        delete $self->{ _insideVcard };
        delete $self->{ _insideId };
        delete $self->{ _insideLanguage };
        delete $self->{ _insideVersion };
        delete $self->{ _insideEntry };
        delete $self->{ _insideContribute }; 
        delete $self->{ _insideDate }; 
        delete $self->{ _insideKeyword };
        delete $self->{ _insideSource };
        delete $self->{ _insideDatetime };
        
        delete $self->{ _insideLangstring };
        delete $self->{ _insideAggregationLevel };
        delete $self->{ _insideCopyrightandotherrestrictions };
        delete $self->{ _insideIdentifier };
        delete $self->{ _insideLearningResourceType };
        delete $self->{ _insideEntity };
        delete $self->{ _insideCentity };        
        delete $self->{ _insideStatus };
        delete $self->{ _insideCatalog };
        delete $self->{ _insideValue };
        delete $self->{ _insideRole };
        delete $self->{ _lang };
    }
    ##### General #####
    elsif ( $tagName eq 'general') {
        # carp 'end_element in general';
        # $self->{ _insideGeneral } = 0;
        my $ob = {
            title => $self->{ lomGeneralTitle } || undef,
            description => $self->{ lomGeneralDescription } || undef,
            identifiers => $self->{ lomGeneralIdentifiers } || undef,
            # keywords => $self->{ lomGeneralKeywords } || undef, # needs to set to undef in general because of x-none vs de vs en which cause an error
            keywords => undef,
            language => $self->{ languages } || DEFAULT_LANGUAGE,
            aggregationLevel => $self->{ lomGeneralAggregationLevel } || undef,
        };
        $self->{ lomGeneral } = $ob;
        $self->{ _insideGeneral } = 0;
    }
    elsif ( $tagName eq 'title' ) {
        if ( $self->{ langstringArray } ) {
            $self->{ lomGeneralTitle } = $self->{ langstringArray };
            delete $self->{ langstringArray };
        } else {
            push( @{ $self->{ lomGeneralTitle } }, $self->{ langstring } ); 
        }
        $self->{ _insideTitle } = 0;
        $self->{ langstring } = '';
    }

    elsif ( $tagName eq 'identifier') {
        my $ent = $self->{ entry };
        if ( $self->{ langstring } ne '' ) {       
            $ent = $self->{ langstring };
            $self->{ langstring } = '';
        }
        my $ob = {
            catalog => $self->{ catalog } || undef,
            entry => trim( $ent )
        };
        push( @{ $self->{ lomGeneralIdentifiers } }, $ob ); 
        $self->{ _insideIdentifier } = 0;
        $self->{ catalog } = '';
    }
    elsif ( $tagName eq 'catalog') {
        # $self->{ _insideCatalog } = 0;
    }

    elsif ( $tagName eq 'keyword') {
        if ( $self->{ langstringArray } ) {
            $self->{ lomGeneralKeywords } = $self->{ langstringArray };
        } else {
            push( @{ $self->{ lomGeneralKeywords } }, $self->{ langstring } );
        }
        $self->{ _insideKeyword } = 0;
        $self->{ langstring } = '';
    }
    elsif ( $tagName eq 'aggregationlevel') {
        my $ob = {
            source => $self->{ lomGeneralAggregationLevelSource } || undef,
            value => $self->{ lomGeneralAggregationLevelValue } || undef
        };
        $self->{ lomGeneralAggregationLevel } = $ob ;
        $self->{ _insideAggregationLevel } = 0;
    }

    ##### Lifecycle #####
    elsif ( $tagName eq 'lifecycle' ) {
        my $ob = {
            version => $self->version() || undef,
            versionDate => $self->{ lomLifecycleVersionDate } || undef,
            status => $self->{ lomLifecycleStatus } || undef,
            contributes => $self->{ lomLifecycleContribute }
        };
        $self->{ lomLifecycle } = $ob;
        $self->{ _insideLifecycle } = 0;
    }
    elsif ( $tagName eq 'status') {
        my $ob = {
            source => $self->{ lomLifecycleStatusSource } || undef,
            value => $self->{ lomLifecycleStatusValue } || undef,
        };
        $self->{ lomLifecycleStatus } = $ob ;
        $self->{ _insideStatus } = 0;
    }

    ##### Metametadata #####
    elsif ( $tagName eq 'metametadata' ) {
        my $ob = {
            contributors => $self->{ lomMetametadataContribute }
        };
        $self->{ lomMetametadata } = $ob;
        $self->{ _insideMetametadata } = 0;
    }

    ##### Technical #####
    elsif ( $tagName eq 'technical' ) {
        my $ob = {
            format => $self->{ format } || 'iMooX',
            size => $self->{ size } || undef,
            location => $self->{ location } || undef,
            duration => $self->{ lomTechnicalDuration } || undef,
            otherplatformrequirements => $self->{ otherplatformrequirements } || undef,
            thumbnail => $self->{ lomTechnicalThumbnail } || undef,
        };
        $self->{ lomTechnical } = $ob;
        $self->{ _insideTechnical } = 0;
    }
    elsif ( $tagName eq 'format ') {
        $self->{ _insideFormat } = 0;
    }
    elsif ( $tagName eq 'size' ) {
        $self->{ _insideSize } = 0;
    }
    elsif ( $tagName eq 'location' ) {
        $self->{ _insideLocation } = 0;
    }
    elsif ( $tagName eq 'duration' ) {
    
        if ( $self->{ _insideDuration } ) {
            my $ob = {
                datetime => $self->{ datetime } || undef,
                description => $self->{ lomDurationDescription } || undef
            };
            $self->{ lomTechnicalDuration } = $ob ;
        }
        $self->{ datetime } = $self->{ lomDurationDescription } = '';
        $self->{ _insideDuration } = 0;
    }
    # non standard element 
    elsif ( $tagName eq 'thumbnail' ) {
        my $ob = {
            url => $self->{ lomTechnicalThumbnailUrl } || undef,
            description => $self->{ lomTechnicalThumbnailDescription } || undef,
            licenses => $self->{ lomTechnicalThumbnailLicenses } || undef
        };
        $self->{ lomTechnicalThumbnail } = $ob;
        $self->{ _insideThumbnail } = 0;
    }
    # non standard element: location of the thumbnail
    elsif ( $tagName eq 'url' ) {
        if (( $self->{ _insideThumbnail } == 1 ) && ( $self->{ _insideLicenses } == 0 )) {
            $self->{ lomTechnicalThumbnailUrl } = $self->{ url };
            $self->{ url } = '';
        }        
        if ( $self->{ _insideLicenses } == 1 ) {
            $self->{ lomTechnicalThumbnailLicensesUrl } = $self->{ url };
            $self->{ url } = '';
        }
        $self->{ _insideUrl } = 0;
    }
    # non standard element: description of the thumbnail
    elsif ( $tagName eq 'licenses' ) {
        my $ob = {
            url => $self->{ lomTechnicalThumbnailLicensesUrl } || undef,
            id => $self->{ id } || undef,
            name => $self->{ name } || undef
        };
        $self->{ lomTechnicalThumbnailLicenses } = $ob;
        $self->{ url } = $self->{ id } = $self->{ name } = '';
        $self->{ _insideLicenses } = 0;
    }

    ##### Educational #####
    elsif ( $tagName eq 'educational' ) {
        my $ob = {
            learningResourceType => $self->{ lomEducationalLearningResourceType } || undef,
            description => $self->{ lomEducationalDescription } || undef,
        };
        $self->{ lomEducational } = $ob;
        $self->{ _insideEducational } = 0;
    }
    elsif ( $tagName eq 'learningresourcetype') {
        my $ob = {
            source => $self->{ lomEducationalLearningResourceTypeSource } || undef,
            id => $self->{ id } || undef,
            entry => $self->{ lomEducationalLearningResourceTypeEntry } || undef,
        };
        push( @{ $self->{ lomEducationalLearningResourceType } }, $ob );
        # $self->{ lomEducationalLearningResourceType } = $ob ;# not just a single object, use hash instead
        $self->{ _insideLearningResourceType } = 0;
    }

    ##### Rights #####
    elsif ( $tagName eq 'rights' ) {   
        my $ob = {
            copyrightandotherrestrictions => $self->{ lomRightsCopyrightandotherrestrictions } || undef,
            description => $self->{ lomRightsDescription } || undef,
        };
        $self->{ lomRights } = $ob;
        $self->{ _insideRights } = 0;
    }
    elsif ( $tagName eq 'copyrightandotherrestrictions') {
        my $ob = {
            source => $self->{ lomRightsCopyrightandotherrestrictionsSource } || undef,
            value => $self->{ lomRightsCopyrightandotherrestrictionsValue } || undef,
        };
        $self->{ lomRightsCopyrightandotherrestrictions } = $ob ;
        $self->{ _insideCopyrightandotherrestrictions } = 0;
    }

    ##### Classification #####
    # IEEE allows multiple classification and UIBK will use two #
    elsif ( $tagName eq 'classification' ) {   
        my $ob = {
            purpose => $self->{ lomClassificationPurpose } || undef,
            taxonPath => $self->{ lomClassificationTaxonpath } || undef,
        };
        push( @{ $self->{ lomClassification } }, $ob );
        $self->{ _insideClassification } = 0;
    }
    elsif ( $tagName eq 'purpose' ) {   
        my $ob = {
            source => $self->{ lomClassificationPurposeSource } || undef,
            value => $self->{ lomClassificationPurposeValue } || undef,
        };
        $self->{ lomClassificationPurpose } = $ob ;
        $self->{ _insidePurpose } = 0;
    }
    elsif ( $tagName eq 'taxonpath' ) {   
        my $ob = {
            source => $self->{ lomClassificationTaxonpathSource } || undef,
            taxon => $self->{ lomClassificationTaxonpathTaxon } || undef,
        };
        push( @{ $self->{ lomClassificationTaxonpath } }, $ob );
        $self->{ _insideTaxonpath } = 0;
    }
    elsif ( $tagName eq 'taxon' ) {   
        my $entry_array = $self->{ lomClassificationTaxonPathTaxonEntry };
        my $ob = {
            id => $self->{ id } || undef,
            entry => $entry_array || undef,
        };
        push( @{ $self->{ lomClassificationTaxonpathTaxon } }, $ob );
        $self->{ _insideTaxon } = 0;
    }

    ##### Used in some parent Elements ####
    elsif ( $tagName eq 'langstring' ) {
        my $xmlLang = $self->{ _lang };
        my $txt = $self->{ langstring };
        if ( $self->{ _lang } ) {
            if (( $self->{ _lang } ne 'x-none' ) and ( $self->{ _lang } ne 'x-t-cc-url' ) and ( $self->{ _lang } ne 'x-t-oefos' ) and ( $self->{ _lang } ne 'x-t-sdg' )) { # o.k. with xml:lang attribute, ignore x-none
                my $obLang = {
                    $xmlLang => $txt
                };
                #$self->{ langstring } = $obLang;
                push( @{ $self->{ langstringArray } }, $obLang );
                $self->{ langstring } = '';
            }
        }
        $self->{ _insideLangstring } = 0;
    }
    elsif ( $tagName eq 'source' ) {
        if ( $self->{ _insideAggregationLevel } == 1 ) {
            $self->{ lomGeneralAggregationLevelSource } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideLearningResourceType } == 1 ) {
            $self->{ lomEducationalLearningResourceTypeSource } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideCopyrightandotherrestrictions } == 1 ) {
            $self->{ lomRightsCopyrightandotherrestrictionsSource } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideMetametadata } == 1 ) {
            $self->{ lomMetametadataContributeRoleSource } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideStatus } == 1 ) {
            $self->{ lomLifecycleStatusSource } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            $self->{ lomLifecycleContributeRoleSource } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insidePurpose } == 1 ) {
            $self->{ lomClassificationPurposeSource } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insideTaxonpath } == 1 ) {
            $self->{ lomClassificationTaxonpathSource } = $self->{ langstring } ;  
        }
        $self->{ _insideSource } = 0;
        $self->{ langstring } = '';
    }
    # used in identifiers or learningResourceTypes
    elsif ( $tagName eq 'entry') {
        if ( $self->{ _insideLearningResourceType } == 1 ) {
            $self->{ lomEducationalLearningResourceTypeEntry } = $self->{ langstringArray };
            delete $self->{ langstringArray } ;
            # push( @{ $self->{ lomEducationalLearningResourceTypeEntry } }, $self->{ langstringArray } );
        }
        elsif ( $self->{ _insideTaxon } == 1 ) {
            $self->{ lomClassificationTaxonPathTaxonEntry } = $self->{ langstringArray };
            delete $self->{ langstringArray } ;
        }
        $self->{ _insideEntry } = 0;
    }
    elsif ( $tagName eq 'description' ) {
        if ( $self->{ _insideGeneral } == 1 ) {
            $self->{ lomGeneralDescription } = $self->{ langstringArray } ;
            delete $self->{ langstringArray } ;
        } 
        elsif ( $self->{ _insideEducational } == 1 ) {
            $self->{ lomEducationalDescription } = $self->{ langstringArray } ;
            delete $self->{ langstringArray } ;
        } 
        elsif ( $self->{ _insideRights } == 1 ) {
            $self->{ lomRightsDescription } = $self->{ langstring } ;
        } 
        elsif ( $self->{ _insideDate } == 1 ) {
            $self->{ lomDateDescription } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideDuration } == 1 ) {
            $self->{ lomDurationDescription } = $self->{ langstring } ;
        }
        $self->{ langstring } = '';
        $self->{ _insideDescription } = 0;
    }
    elsif ( $tagName eq 'value' ) {
        if ( $self->{ _insideAggregationLevel } == 1 ) {
            $self->{ lomGeneralAggregationLevelValue } = $self->{ langstring } ;
        }
        elsif ( $self->{ _insideCopyrightandotherrestrictions } == 1 ) {
            $self->{ lomRightsCopyrightandotherrestrictionsValue } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insideMetametadata } == 1 ) {
            $self->{ lomMetametadataContributeRoleValue } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insideStatus } == 1 ) {
            $self->{ lomLifecycleStatusValue } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            $self->{ lomLifecycleContributeRoleValue } = $self->{ langstring } ;  
        }
        elsif ( $self->{ _insideClassificationPurpose } == 1 ) {
            $self->{ lomClassificationPurposeValue } = $self->{ langstring } ;  
        }
        $self->{ _insideValue } = 0;
        $self->{ langstring } = '';
    }
    # used in metametadata and lifecycle
    elsif ( $tagName eq 'contribute' ) {
        if ( $self->{ _insideMetametadata } == 1 ) {
            my $ob = {
                role => $self->{ lomMetametadataContributeRole } || undef,
                entities => $self->{ lomMetametadataContributeEntity } || undef,
                centities => $self->{ lomMetametadataContributeCentity } || undef,
                date => $self->{ lomMetametadataContributeDate } || undef
            };
            push( @{ $self->{ lomMetametadataContribute } }, $ob );
            $self->{ lomMetametadataContributeDate } = undef;
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            my $ob = {
                role => $self->{ lomLifecycleContributeRole } || undef,
                entities => $self->{ lomLifecycleContributeEntity } || undef,
                centities => $self->{ lomLifecycleContributeCentity } || undef
            };
            push( @{ $self->{ lomLifecycleContribute } }, $ob );
        }
        $self->{ _insideContribute } = 0;
    }
    elsif ( $tagName eq 'role') {
        if ( $self->{ _insideMetametadata } == 1 ) {
            my $ob = {
                source => $self->{ lomMetametadataContributeRoleSource } || undef,
                value => $self->{ lomMetametadataContributeRoleValue } || undef
            };
            $self->{ lomMetametadataContributeRole } = $ob ;
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            my $ob = {
                source => $self->{ lomLifecycleContributeRoleSource } || undef,
                value => $self->{ lomLifecycleContributeRoleValue } || undef
            };
            $self->{ lomLifecycleContributeRole } = $ob ;
        }
        $self->{ _insideRole } = 0;
    }
    elsif ( $tagName eq 'entity') {
        if ( $self->{ _insideMetametadata } == 1 ) {  
            # old implmentation did not use vcard        
            push( @{ $self->{ lomMetametadataContributeEntity } }, $self->{ entity } );    
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            # old implmentation did not use vcard          
            push( @{ $self->{ lomLifecycleContributeEntity } }, $self->{ entity } );
            $self->{ vcard } = '';
        }
        $self->{ entity } = '';
        $self->{ _insideEntity } = 0;
    }
    elsif ( $tagName eq 'centity') {
        if ( $self->{ _insideMetametadata } == 1 ) {
            my $ob = {
                vcard => $self->{ vcard } || undef
            };
            push( @{ $self->{ lomMetametadataContributeCentity } }, $ob );      
        }
        elsif ( $self->{ _insideLifecycle } == 1 ) {
            my $ob = {
                vcard => $self->{ vcard } || undef
            };
            push( @{$self->{ lomLifecycleContributeCentity } }, $ob );  
            $self->{ vcard } = '';
        }
        $self->{ _insideCentity } = 0;
    }    
    elsif ( $tagName eq 'date') {
        if ( $self->{ _insideDate } ) {
            my $ob = {
                datetime => $self->{ datetime } || undef,
                description => $self->{ lomDateDescription } || undef
            };
            $self->{ lomMetametadataContributeDate } = $ob ;
        }
        $self->{ _insideDate } = 0;
        $self->{ datetime } = $self->{ lomDateDescription } = '';
    }
    elsif ( $tagName eq 'version') {
        # a new implementation style - is it working?
        $self->version($self->{ langstring });
        $self->{ _insideVersion } = 0;
        $self->{ langstring } = '';
    }
    elsif ( $tagName eq 'vcard' ) {
        $self->{ _insideVcard } = 0;
    }    
    elsif ( $tagName eq 'language' ) {
        push( @{$self->{ languages } }, $self->{ language } );
        $self->{ _insideLanguage } = 0;
        $self->{ language } = '';
    }
    elsif ( $tagName eq 'datetime' ) {
        if ( $self->{ _insideLifecycle } == 1 ) {
            $self->{ lomLifecycleVersionDate } = $self->{ datetime } || undef;
        }
        $self->{ datetime } = '';
        $self->{ _insideDatetime } = 0;
    }
    elsif ( $tagName eq 'format' ) {
        $self->{ _insideFormat } = 0;
    }    
    elsif ( $tagName eq 'id' ) {
        $self->{ _insideId } = 0;
    }
    elsif ( $tagName eq 'metadata' ) {
        delete $self->{ _insideMetadata };;
        delete $self->{ _tagStack };
        delete $self->{ _insideLom };
    }
    else {
        carp "who am I EE? ($tagName)";
        #return $self->SUPER::end_element( $element );
    };
    #return $self->SUPER::end_element($element) if $self->{ fwdAll };
}

sub ignorable_whitespace {
    my ( $self, $characters ) = @_;
    return $self->SUPER::ignorable_whitespace( $characters ) if $self->{ _insideLom };
}


sub characters {
    my ( $self, $characters ) = @_;
    $self->{ $self->{ _tagStack }[-1] } .= $characters->{ Data } if $self->{ _insideLom };
}
 
1;
