package App::Type::LOMIngest_general;

use strict;
use warnings;

our (@ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $VERSION);
use Exporter;

$VERSION = 0.01;
@ISA = qw(Exporter);
@EXPORT = qw(getId getAbstract getLangAbstract getAbstractMostRelated getTitle getTitleMostRelated getTitleLangList getLangTitle getLanguageList getDirectLink getLicense getCreated getOefosList getOefosId onlyOefosSubCategories getFormat getTechnicalFormat getLearningRTList getAuthors getPublishers getThumbnailUrl ingestTimeStamp today);
use FindBin;
use lib "$FindBin::Bin/../../";

# print "I am in package: ".__PACKAGE__."\n";

sub getId {
  my ($lomData, $catalog) = @_;
  my $general = $lomData->{general};
  for my $identify  ( @{$general->{identifiers}} ) {
    if ( $identify->{catalog} eq $catalog ) {
      return $identify->{entry};
    }
  }
  return;
}

sub getAbstract {
  my ($lomData, $lang) = @_;
  my $general = $lomData->{general};
  $lang = $lang || 'de';
  for my $abstract  ( @{$general->{description}} ) {
    return $abstract->{$lang};
  }
}

sub getLangAbstract {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my %langabstract = ();
  for my $abstract  ( @{$general->{description}} ) {
    foreach my $lang (keys %$abstract) {
      $langabstract{$lang} = $abstract->{$lang};
    }
  }
  return \%langabstract;
}

sub getAbstractMostRelated {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my %abstractlang = ();
  my %abstractlangsort = ();
  my $default = 'de';
  for my $abstract  ( @{$general->{description}} ) {
    my $abstract_type = ref($abstract);
    if ( $abstract_type eq "HASH" ) {
      foreach my $lang (keys %$abstract) {
        $abstractlang{$lang} = $abstract->{$lang};
      }
    } else {
      $abstractlang{$default} = $abstract;
    }
  }
  # de, en , es ....
  foreach my $tl (sort keys %abstractlang) {
    if ( length($abstractlang{$tl}) > 0) {
      return $abstractlang{$tl};
    }
  }
}

sub getTitle {
  my ($lomData, $lang) = @_;
  my $general = $lomData->{general};
  $lang = $lang || 'de';
  for my $title  ( @{$general->{title}} ) {
    return $title->{$lang};
  }
}

sub getTitleMostRelated {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my %titlelang = ();
  my %titlelangsort = ();
  my $default = 'de';
  for my $titles  ( @{$general->{title}} ) {
    my $titles_type = ref($titles);
    if ( $titles_type eq "HASH" ) {
      foreach my $lang (keys %$titles) {
        $titlelang{$lang} = $titles->{$lang};
      }
    } else {
      $titlelang{$default} = $titles;
    }
  }
  # de, en , es ....
  foreach my $tl (sort keys %titlelang) {
    if ( length($titlelang{$tl}) > 0) {
      return $titlelang{$tl};
    }
  }
}

sub getTitleLangList {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my @titlelangs = ();
  for my $titles  ( @{$general->{title}} ) {
      foreach my $lang (keys %$titles) {
        push(@titlelangs, $lang);
      }
  }
  return \@titlelangs;
}

sub getLangTitle {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my %langtitle = ();
  my $default = 'de';
  for my $titles  ( @{$general->{title}} ) {
    my $titles_type = ref($titles);
    if ( $titles_type eq "HASH" ) {
      foreach my $lang (keys %$titles) {
        $langtitle{$lang} = $titles->{$lang};
      }
    } else {
      $langtitle{$default} = $titles;
    }
  }
  return \%langtitle;
}

sub getLanguageList {
  my ($lomData) = @_;
  my $general = $lomData->{general};
  my @languages = ();
  for my $language  ( @{$general->{language}} ) {
    push(@languages, $language);
  }
  return \@languages;
}

sub getDirectLink {
  my ($lomData) = @_;
  my $technical = $lomData->{technical};
  return $technical->{location};
}

sub getThumbnailUrl {
  my ($lomData) = @_;
  my $technical = $lomData->{technical};
  return $technical->{thumbnail}->{url};
}

sub getLicenseOld {
  my ($lomData) = @_;
  my $rights = $lomData->{rights};
  my $linkCC = $rights->{description};

  my $txt = '';
  if ( ($txt) = $linkCC =~ /^https\:\/\/creativecommons.org\/licenses\/(.*)/ ) {
      $txt =~ s/\// /g;
      # remove trailing space
      $txt =~ s/\s+$//;
      my $upperCaseTxt = uc $txt;
      if ( $upperCaseTxt !~ /(.*)NULL$/ ) {
        $txt = 'CC '.$upperCaseTxt;
      } else {
        $txt = 'ERROR: BAD CC License';
      }
  }

  return $txt;
}

sub _getLicenseCC {
  my ($lomData) = @_;
  my $rights = $lomData->{rights};
  my $linkCC = defined $rights->{description} ? $rights->{description} : '' ;

  my $txt = '';
  if ( ($txt) = $linkCC =~ /^https\:\/\/creativecommons.org\/licenses\/(.*)/ ) {
      # remove trailing slash
      $txt =~ s/\/+$//;
      # change 
      $txt =~ s/\//-/g;
      my $upperCaseTxt = uc $txt;
      if ( $upperCaseTxt !~ /(.*)NULL$/ ) {
        $txt = 'CC-'.$upperCaseTxt;
      } else {
        $txt = 'ERROR: NULL CC License';
      }
  }

  return $txt;
}

sub _getLicenseSpdx {
  my ($lomData) = @_;
  my $rights = $lomData->{rights};
  my $linkSpdx = defined $rights->{description} ? $rights->{description} : '';

  my $txt = '';
  if ( ($txt) = $linkSpdx =~ /^https\:\/\/spdx.org\/licenses\/(.*)/ ) {
      # remove trailing space
      $txt =~ s/\s+$//;
      if ( $txt =~ /(.*)NULL$/ ) {
        $txt = 'ERROR: NULL SPDX License';
      }
  }

  return $txt;
}

sub getLicense {
  my ($lomData) = @_;

  my $txt = '';
  my $licenseCC = _getLicenseCC($lomData);
  my $licenseSpdx = _getLicenseSpdx($lomData);
  if ( defined $licenseCC and length $licenseCC ){
      $txt = $licenseCC;
  }
  elsif ( defined $licenseSpdx and length $licenseSpdx ){
      $txt = $licenseSpdx;
  }

  return $txt;
}


sub getCreated {
  my ($lomData) = @_;
  my $lifecycle = $lomData->{lifecycle};
  my $date = $lifecycle->{versionDate} || '';
  if ( my ($year, $mon, $day, $tim) = ($date =~ /^(\d{4})-(\d{2})-(\d{2})(.*)/) ) {
    return $year.'-'.$mon.'-'.$day;
  }
  return today();
}

sub getOefosList {
    my ($lomData) = @_;
    # source may look like: 'https://w3id.org/oerbase/vocabs/oefos2012', 'https://oer-portal.uni-graz.at/vocabs/oefos2012'
    my $id = '';
    my $oefosId = '';
    my @oefosList = ();

    for my $classification  ( @{$lomData->{classification}} ) {
      for my $taxonPath  ( @{$classification->{taxonPath}} ) {
        if ( $taxonPath->{source} =~ /(.*)\/vocabs\/oefos2012$/ ) {
          for my $taxon  ( @{$taxonPath->{taxon}} ) {
            $id = $taxon->{id};
            $oefosId = getOefosId($id);
            if ( length($oefosId) > 0 ){
                # do acceppt all levels (some repos like uni-graz have just one digit for discipline like SOZIALWISSENSCHAFTEN (5))
                push(@oefosList, $oefosId );                    
            }
          }
        }
      }
    }
    return \@oefosList;
}

sub getOefosId {
    my ($id) = @_;
    # vocabs may look like: 'https://w3id.org/oerbase/vocabs/oefos2012/', 'https://oer-portal.uni-graz.at/vocabs/oefos2012/'
    my $number = '';
    my $prefixurl = '';
    if ( ($prefixurl, $number) = $id =~ /(.*)\/vocabs\/oefos2012\/(.*)/ ) {
      return $number;
    }
    return '';
}


sub onlyOefosSubCategories {
    my ($inlist) = @_;
    my $found = 0;
    my $restlist = $inlist;
    my @onlySubelements = ();
    for my $i ( @$inlist ) {
        $found = 0;
        for my $s ( @$restlist ) {
            if ($s ne $i) {
                if ( $s =~ /^$i/ ) {
                    $found = 1;
                    last;
                } else {
                    $found = 0;
                }
            }
        }
        if ($found == 0) {
            push @onlySubelements, $i;
        }
    }
    return \@onlySubelements;
}

sub getFormat {
  my ($lomData) = @_;
  my $technical = $lomData->{technical};
  my $resourceFormat;
  if ( defined $technical->{format} ) {
    for ( $technical->{format} ) {
      if (/^audio\/mp3$/) {
        $resourceFormat = 'Audio';
      }
      elsif (/^application\/pdf$/) {
        $resourceFormat = 'Document';
      }
      elsif (/^image\/png$/) {
        $resourceFormat = 'Picture';
      }
      elsif (/^video\/mp4$/) {
        $resourceFormat = 'Video';
      }
      else {
        $resourceFormat = 'Miscellaneous';
      }
    }
  }
  else {
    $resourceFormat = 'unknown';
  }
  return $resourceFormat;
}

sub getTechnicalFormat {
  my ($lomData) = @_;
  my $technical = $lomData->{technical};
  my $technicalFormat;
  $technicalFormat = defined $technical->{format} ? $technical->{format} : '';
  ## check in a List if format is allowed t.b.impl
  if (length($technicalFormat) gt 0) {
    $technicalFormat = ( split '/', $technicalFormat )[ -1 ];
    $technicalFormat = ( split '\.', $technicalFormat )[ -1 ];
  }
  else {
    $technicalFormat = 'unknown';
  }
  return $technicalFormat;
}

sub getLearningRTList {
    my ($lomData) = @_;
    # source may look like: 'https://w3id.org/kim/hcrt/scheme'
    my $id = '';
    my $learningRTId = '';
    my @learningRTList = ();

    my $educational = $lomData->{educational};
    for my $learningresourcetype  ( @{$educational->{learningResourceType}} ) {
      if ( $learningresourcetype->{source} =~ /(.*)\/kim\/hcrt\/scheme$/ ) {
        $id = $learningresourcetype->{id};
        $learningRTId = getLearningRTId($id);
        if ( length($learningRTId) > 0 ){
          push(@learningRTList, $learningRTId );
        }
      }
    }
    return \@learningRTList;
}

sub getLearningRTId {
    my ($id) = @_;
    # vocabs may look like: 'https://w3id.org/kim/hcrt/video'
    my $number = '';
    my $prefixurl = '';
    if ( ($prefixurl, $number) = $id =~ /(.*)\/kim\/hcrt\/(.*)/ ) {
      return $number;
    }
    return '';
}


sub getAuthors {
  my ($lomData) = @_;
  my $role = 'Author';
  my $lifecycle = $lomData->{lifecycle};
  my $fullName = '';
  my @authors = ();
  for my $contribute  ( @{$lifecycle->{contributes}} ) {
    if ( $contribute->{role}->{value} eq $role ) {
      for my $entity  ( @{$contribute->{entities}} ) {
        $fullName = $entity;
        if (length($fullName) gt 0) {
          push(@authors, $fullName);
        }
      }
    }
  }
  return \@authors;
}

sub getPublishers {
  my ($lomData) = @_;
  my $role = 'Publisher';
  my $lifecycle = $lomData->{lifecycle};
  my $fullName = '';
  my @publishers = ();
  for my $contribute  ( @{$lifecycle->{contributes}} ) {
    if ( $contribute->{role}->{value} eq $role ) {
      for my $entity  ( @{$contribute->{entities}} ) {
        $fullName = $entity;
        if (length($fullName) gt 0) {
          push(@publishers, $fullName);
        }
      }
    }
  }
  return \@publishers;
}

sub ingestTimeStamp {
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
  my $timestamp = sprintf ( "%04d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
  return $timestamp;
}

sub today {
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
  my $timestamp = sprintf ( "%04d-%02d-%02d", $year+1900,$mon+1,$mday);
  return $timestamp;
}


1;

__END__
