# package App::Type::LOMIngest_invenio;

use strict;
use warnings;

our (@ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $VERSION);
use Exporter;

$VERSION = 0.02;
@ISA = qw(Exporter);
@EXPORT = qw(getDOILink getId getAbstract getLangAbstract getAbstractMostRelated getTitle getTitleMostRelated getTitleLangList getLangTitle getLanguageList getDirectLink getLicense getCreated getOefosList getOefosId onlyOefosSubCategories getAuthors getPublishers getFormat getTechnicalFormat getLearningRTList getThumbnailUrl ingestTimeStamp today);

use FindBin;
use lib "$FindBin::Bin/../../";

use App::Type::LOMIngest_general;

print "I am in package: ".__PACKAGE__."\n";

# additional function
sub getDOILink {
  my ($lomData) = @_;
  my $doiLink= 'https://doi.org/';
  my $catalog = "DOI";
  return $doiLink .getId($lomData, $catalog);
}

# overrides getFormat from LOMIngest_general
sub getFormat {
  my ($lomData) = @_;
  my $technical = $lomData->{technical};
  my $resourceFormat;
  for ( $technical->{format} ) {
    if (/^audio\/mp3$/) {
      $resourceFormat = 'Audio';
    }
    elsif (/^application\/pdf$/) {
      $resourceFormat = 'Document';
    }
    elsif (/^image\/png$/) {
      $resourceFormat = 'Picture';
    }
    elsif (/^video\/mp4$/) {
      $resourceFormat = 'Video';
    }
    elsif (/^iMooX$/) {
      $resourceFormat = 'iMooX';
    }
    else {
      $resourceFormat = 'Miscellaneous';
    }
  }
  return $resourceFormat;
}

# overrides getAuthors from LOMIngest_general
sub getAuthors {
  my ($lomData) = @_;
  my $role = 'Author';
  my $lifecycle = $lomData->{lifecycle};
  my $fullName = '';
  my $org = '';
  my $author = '';
  my @authors = ();
  for my $contribute  ( @{$lifecycle->{contributes}} ) {
    if ( $contribute->{role}->{value} eq $role ) {
      for my $centity  ( @{$contribute->{centities}} ) {
        $fullName = getFullName($centity->{vcard});
        $org = getOrg($centity->{vcard});
        if ( length($fullName) > 0 ) {
          if ( length($org) > 0 ) {
            $author = $fullName.' ('.$org.')';
          } else {
            $author = $fullName;
          }
        } else {
          $author = $org;
        }
        if (length($author) gt 0) {
          push(@authors, $author);
        }
      }
    }
  }
  return \@authors;
}

sub getPublishers {
  my ($lomData) = @_;
  my $role = 'Publisher';
  my $lifecycle = $lomData->{lifecycle};
  my $fullName = '';
  my $org = '';
  my $publisher = '';
  my @publishers = ();
  for my $contribute  ( @{$lifecycle->{contributes}} ) {
    if ( $contribute->{role}->{value} eq $role ) {
      for my $centity  ( @{$contribute->{centities}} ) {
        $fullName = getFullName($centity->{vcard});
        $org = getOrg($centity->{vcard});
        if ( length($fullName) > 0 ) {
          if ( length($org) > 0 ) {
            $publisher = $fullName.' ('.$org.')';
          } else {
            $publisher = $fullName;
          }
        } else {
          $publisher = $org;
        }
        if (length($publisher) gt 0) {
          push(@publishers, $publisher);
        }
      }
    }
  }
  return \@publishers;
}

# private 
sub getFullName {
    my ($vcard) = @_;
    my $fn = '';
    my @lines = split('\n', $vcard);
    my $line;
    foreach $line ( @lines ) {
        if ( ($fn) = $line =~ /^FN:(.*)/ ) {
            return $fn;
        }
    }
}

sub getOrg {
    my ($vcard) = @_;
    my $org = '';
    my @lines = split('\n', $vcard);
    my $line;
    foreach $line ( @lines ) {
        if ( ($org) = $line =~ /^ORG:(.*)/ ) {
            $org =~ s/\\//g;
            return $org;
        }
    }
}

1;

__END__
