package App::Type::Vocabs;

use strict;
use warnings;

use Carp;
use Data::Dumper;

use FindBin;
use lib "$FindBin::Bin/../../";
use Encode;
use App::Logger;
use utf8;
use JSON::XS;
use HTTP::Request;
use LWP::UserAgent;

sub new {
  my ($class, %args) = @_;

  my $self = {};
  # mandatory
  $self->{_index} = $args{vocabs} || croak "no vocabs given";

  bless($self, $class);

  $self->_init();

  return $self;
}

sub _init {
  my ($self) = @_;
  croak "_init is private" if caller() ne __PACKAGE__;
  # config
  my $config = YAML::Syck::LoadFile("$FindBin::Bin/etc/sources/vocabs/config.yml");
  my $index = $self->{_index};
  my $cfg = $config->{vocabs}->{ $index }->{cfg};
  # print "show cfg " . Dumper( $cfg ) ;
  # print "show debug " . Dumper( $cfg->{debug} ) ;
  # print "show path " . Dumper( $cfg->{log}->{path} ) ;
  # print "show file " . Dumper( $cfg->{log}->{filename} ) ;

  # new logger
  $self->{_logger} = App::Logger->new(
    file => $FindBin::Bin.'/'.$cfg->{log}->{path}.'/'.$cfg->{log}->{filename},
    debug => defined $cfg->{debug} ? $cfg->{debug} : 1,
    verbosity => defined $cfg->{verbosity} ? $cfg->{verbosity} : 2,
  );


  $self->{_logger}->debug('cfg ' . Dumper( $cfg ) );
  $self->{_uri} = defined $cfg->{uri} ? $cfg->{uri}->{scheme} . ':' . $cfg->{uri}->{authority} . $cfg->{uri}->{path} : 'http://index:9200/' . $index;

  return;
}

sub getVocab {
  my ($self, $id, $what) = @_;
  if ( $id ne "" ) {
    my $uri = $self->{_uri} . "/_doc/$id";
    $self->{_logger}->info('uri |'.$uri.'|', 2);
    my $req = HTTP::Request->new( 'GET', $uri );
    my $ua = LWP::UserAgent->new;
    # $req->header( 'Content-Type' => 'application/json' );
    my $source;
    my $res = $ua->request( $req );

    if ( !$res->is_success() ) {
      $self->{_logger}->fatal("no valid response from uri |$uri|, status_line |".$res->status_line()."|, content |".$res->content()."|");
      return "";
    } else {
      my $body = decode_json($res->content());
      my $source = $body->{_source};
      $self->{_logger}->info('res status_line |'.$res->status_line().'|', 2);
      if ( $what ne "" ) {
        return $source->{$what};
      } else {
        $self->{_logger}->info('processing id[' . $id . '] skipped, reason unknown what!', 1);
        return "";
      }
    }
  } else {
    $self->{_logger}->info('processing id[] skipped, reason unknown id!', 1);
    return "";
  }
}

1;

__END__
