#!/bin/bash

export DOCKER_REGISTRY=localhost:5443
export DOCKER_BUILDDATE=`date +%Y-%m-%d`
OEA_TAG=current
OEA_VERSIONNR=1.0.0.00-ce
OEA_SERVICE=oeaplatform

rebuild() {
  local OEA_IMAGE=$1
  allowedImages="stats metaingest metaharvest index frontend-client frontend-server webserver mq"
  for allowedImage in $allowedImages ; do
    if [ "$OEA_IMAGE" == "$allowedImage" ] 
    then
      echo "Start proceeding Image \"$OEA_IMAGE\" ..."
      docker stack rm oeaplatform
      TAG=`date +%Y-%m-%dT%H.%M.%S` && docker compose -f docker-compose.yml build --no-cache --pull $OEA_IMAGE
      ID=`docker images -q $DOCKER_REGISTRY/$OEA_IMAGE:$OEA_TAG`
      docker tag $ID $OEA_IMAGE:$TAG
      docker push $DOCKER_REGISTRY/$OEA_IMAGE:$OEA_TAG
      docker tag $ID $DOCKER_REGISTRY/$OEA_IMAGE:$OEA_VERSIONNR
      docker push $DOCKER_REGISTRY/$OEA_IMAGE:$OEA_VERSIONNR
      docker stack deploy --detach=true --prune --with-registry-auth -c docker-compose.yml $OEA_SERVICE
    fi
  done
  echo "Finished"
}

purge() {
  docker rmi $(docker images -f "dangling=true" -q)
  docker rmi --force $(docker images|grep none|awk '{print $3}')
}

usage() {
  echo "Use: [OPTION] <image_name>"
  echo "Possible options are: "
  echo "     -h: this help"
  echo "     -l: list of images"
  echo "     -b <image_name>: build the image"
  echo "     -r: removes images with tag <none> and cleans dangling images"
}

while getopts ':hlrb:' OPTION ; do
  case "$OPTION" in
    b)   rebuild $OPTARG ;;
    r)   purge;;
    h)   usage;;
    l)   echo "stats (=opensearch-dashboards), metaingest, metaharvest, index (=opensearch), frontend-client, frontend-server, webserver, mq";;
    *)   echo "Unknown parameter"
  esac
done
